#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to start or stop service for TMNAS."""

import os
import stat
import pwd
import shutil

import ionas.request as request
import ionas.confut as confut
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SERVICECTRL_PATH, Results, ReqTypes)
from product_conf import (PACKAGES_DIR, SYS_ETC_DIR)
from product_conf.devinfo import (VOL_INFO,)

EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
SYSTEM_CATEGORY = "system"
MOUNT_ACTION = "mount"
UNMOUNT_ACTION = "unmount"
SHUTDOWN_ACTION = "shutdown"
REBOOT_ACTION = "reboot"

SPLX_SERVICE = "splx"
SYSLOG_SERVICE = "syslog-ng"

NO_SUFFIX = ("",)
SYSLOG_SPLX_SRC = os.path.join(
    PACKAGES_DIR, 'tmnas', 'syslog-ng.conf')
SYSLOG_SPLX_DST = os.path.join(
    SYS_ETC_DIR, 'syslog-ng', 'conf.d', 'splx.conf')

INTERNAL_MNT_POINT = VOL_INFO[config.INTERNAL_VOL]["mnt_point"]
LOG_DIR = os.path.join("var", "log")
OPT_DIR = os.path.join("var", "opt")
TREND_MICRO_DIR = "TrendMicro"
HTTPD_DIR = "httpd"
LOGS_DIR = "logs"
NORMAL_DIR_PERM = (stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
                        stat.S_IROTH | stat.S_IXOTH)
TREND_MICRO_DIR_PERM = stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO
TMHTTPD_USER_NAME = "tmhttpd"


SERVICECTRL_REQ_XML_TMPL = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
"""


class EvhTmNasService(evhcmd.EvhCmd):
    """Event handler to start or stop services for TMNAS."""

    def __mkdir(self, path, mode):
        """Make "path" directory including the parent directories."""

        try:
            if not os.path.exists(path):
                os.makedirs(path, mode)
        except:
            # Ignore error
            pass

    def __prepair_dirs(self, mnt_point):
        """Prepair directories for TMANS."""

        log_dir = os.path.join(mnt_point, LOG_DIR)
        log_tm_dir = os.path.join(log_dir, TREND_MICRO_DIR)
        opt_dir = os.path.join(mnt_point, OPT_DIR)
        opt_tm_dir = os.path.join(opt_dir, TREND_MICRO_DIR)
        opt_tm_httpd_dir = os.path.join(opt_tm_dir, HTTPD_DIR)
        opt_tm_httpd_logs_dir = os.path.join(opt_tm_httpd_dir, LOGS_DIR)

        pwd_user = pwd.getpwnam(TMHTTPD_USER_NAME)
        uid = pwd_user.pw_uid
        gid = pwd_user.pw_gid

        self.__mkdir(log_dir, NORMAL_DIR_PERM)
        self.__mkdir(log_tm_dir, TREND_MICRO_DIR_PERM)

        self.__mkdir(opt_dir, NORMAL_DIR_PERM)
        self.__mkdir(opt_tm_dir, TREND_MICRO_DIR_PERM)
        self.__mkdir(opt_tm_httpd_dir, NORMAL_DIR_PERM)
        self.__mkdir(opt_tm_httpd_logs_dir, TREND_MICRO_DIR_PERM)
        os.chown(opt_tm_httpd_logs_dir, uid, gid)

    def __send_request(self, target, type, body):
        """Send request entry to "target"."""

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __service_control(self, name, type):
        """Control service."""

        self.__send_request(FAL_SERVICECTRL_PATH, type,
                        SERVICECTRL_REQ_XML_TMPL % (name,))

    def __update_syslog_conf(self, is_tmnas_enabled):
        """Update a configuration file for syslog."""

        if is_tmnas_enabled:
            try:
                shutil.copy(SYSLOG_SPLX_SRC, SYSLOG_SPLX_DST)
            except:
                self.exception("Fail enabling %s" % SYSLOG_SPLX_DST)
        else:
            try:
                if os.path.exists(SYSLOG_SPLX_DST):
                    os.remove(SYSLOG_SPLX_DST)
            except:
                self.exception("Fail disabling %s" % SYSLOG_SPLX_DST)

    def __handle_volume_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):

            # Non target events
            entry.set_result(Results.OK)
            return entry

        if action == MOUNT_ACTION:
            if not is_previous:
                self.__prepair_dirs(INTERNAL_MNT_POINT)
                self.__service_control(SYSLOG_SERVICE, ReqTypes.STOP)
                self.__update_syslog_conf(True)
                self.__service_control(SYSLOG_SERVICE, ReqTypes.START)
                self.__service_control(SPLX_SERVICE, ReqTypes.START)

        elif action == UNMOUNT_ACTION:
            if is_previous:
                self.__service_control(SPLX_SERVICE, ReqTypes.STOP)
                self.__service_control(SYSLOG_SERVICE, ReqTypes.STOP)
                self.__update_syslog_conf(False)
                self.__service_control(SYSLOG_SERVICE, ReqTypes.START)

        entry.set_result(Results.OK)
        return entry

    def __handle_system_event(self, eventinfo, entry):
        """Handle system event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if action in (SHUTDOWN_ACTION, REBOOT_ACTION) and is_previous:
            self.__service_control(SPLX_SERVICE, ReqTypes.STOP)
            self.__service_control(SYSLOG_SERVICE, ReqTypes.STOP)
            self.__update_syslog_conf(False)
            self.__service_control(SYSLOG_SERVICE, ReqTypes.START)

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volume_event(eventinfo, entry)

            if eventinfo.get_attr_value(CATEGORY_ATTR) == SYSTEM_CATEGORY:
                return self.__handle_system_event(eventinfo, entry)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhTmNasService(option=evhcmd.EvhCmdOption()).handle_request()
