#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for installing the TMNAS."""

import os

import iobase.xmldata as xmldata
import iobase.cmdexec as cmdexec
import ionas.request as request
from product_conf import (BIN_DIR, LIBEXEC_DIR, PACKAGES_DIR, XML_CONF_DIR,
                            STOR_IF_TO_VOL, )
from fal_info import (FAL_VOLCTRL_PATH, Results, ReqTypes)
import fal_msginfo

import manualctrl


ENCODING = "UTF-8"
CATEGORYLIST_PATH = "categorylist"
CATEGORY_NAME = "category"
CATEGORY_KEY_PATH = "code"
CATEGORY_FIND_XPATH_FMT = """category[code="%s"]"""

MESSAGEINFOLIST_PATH = "messageinfolist"
MESSAGEINFO_NAME = "messageinfo"
MESSAGEINFO_KEY_PATH = "code"
MESSAGEINFO_FIND_XPATH_FMT = """messageinfo[code="%s"]"""

MESSAGES_FILE = os.path.join(XML_CONF_DIR, "messages.xml")
PACKAGE_MESSAGES_FILE = os.path.join(PACKAGES_DIR, "tmnas", "messages.xml")
BUILD_MSGINFO_CMD = os.path.join(BIN_DIR, "build_msginfo")

PYC_SUFFIX = ".pyc"
PY_SUFFIX = ".py"

HANDLER_FOR_SHARE = os.path.join(LIBEXEC_DIR, "evh", "evh_tmnas_share.py")
HANDLER_FOR_SERVICE = os.path.join(LIBEXEC_DIR, "evh", "evh_tmnas_service.py")
INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]

MOUNT_EVENT_XML = '<events><eventinfo category="volume" action="mount">' + \
                    '<volume><name>' + INTERNAL_VOL + '</name></volume>' + \
                    '</eventinfo></events>'

VOLUME_QUERY_REQ_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

VOLUME_STATUS_PATH = "volume/status"
STATUS_MOUNTED = "mounted"


def __send_request(target, type, body):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def __merge_list(new_messages_xml, messages_xml, list_xpath,
        item_name, key_in_item_xpath, find_xpath_fmt):
    """Merge a list in "new_mssages_xml" into "messages_xml"."""

    new_list_node = new_messages_xml.get_node(list_xpath)
    list_node = messages_xml.get_node(list_xpath)

    for item_node in new_list_node.get_node_list(item_name):
        key = item_node.get_value(key_in_item_xpath)
        try:
            cur_node = list_node.get_node(find_xpath_fmt % (key,))
        except:
            list_node.append_child(item_node.get_clone())


def __merge_categorylist(new_messages_xml, messages_xml):
    """Merge "categorylist" tree in "new_mssages_xml" into "messages_xml"."""

    __merge_list(new_messages_xml, messages_xml, CATEGORYLIST_PATH,
        CATEGORY_NAME, CATEGORY_KEY_PATH, CATEGORY_FIND_XPATH_FMT)


def __merge_messageinfolist(new_messages_xml, messages_xml):
    """Merge "messageinfolist" tree in "new_mssages_xml"
       into "messages_xml".
    """

    __merge_list(new_messages_xml, messages_xml, MESSAGEINFOLIST_PATH,
        MESSAGEINFO_NAME, MESSAGEINFO_KEY_PATH, MESSAGEINFO_FIND_XPATH_FMT)


def __update_msginfo(messages_file):
    """Update a "fal_msginfo" for "message_file"."""

    fal_msginfo_path = fal_msginfo.__file__

    (base, suffix) = os.path.splitext(fal_msginfo_path)
    if suffix == PYC_SUFFIX:
        fal_msginfo_path = base + PY_SUFFIX

    cmd_args = [BUILD_MSGINFO_CMD, messages_file, fal_msginfo_path]
    cmdexec.CmdExec(cmd_args).execute()


def update_messages(messages_file, package_messages_file):
    """Update "messages_file" by "package_messages_file"."""

    messages_xml = xmldata.XmlConf(messages_file, encoding=ENCODING)
    package_messages_xml = xmldata.XmlConf(package_messages_file,
                                encoding=ENCODING)

    __merge_categorylist(package_messages_xml, messages_xml)
    __merge_messageinfolist(package_messages_xml, messages_xml)
    tmp_path = messages_file + "." + str(os.getpid())
    messages_xml.save(tmp_path)
    os.rename(tmp_path, messages_file)

    __update_msginfo(messages_file)


def is_volume_mounted(vol):
    """Check if "vol" is mounted."""

    result_entry = __send_request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                            VOLUME_QUERY_REQ_XML_TMPL % (vol,))

    if (result_entry.get_result() == Results.OK and
        result_entry.get().get_value(VOLUME_STATUS_PATH) == STATUS_MOUNTED):

        return True

    return False


def create_tmadmin_share():
    """Create "tmadmin" share folder by a event handler."""

    __send_request(HANDLER_FOR_SHARE, ReqTypes.EVENT, MOUNT_EVENT_XML)


def start_tmnas_service():
    """Start service for the TMNAS by a event handler."""

    __send_request(HANDLER_FOR_SERVICE, ReqTypes.EVENT, MOUNT_EVENT_XML)


#
# Main
#

update_messages(MESSAGES_FILE, PACKAGE_MESSAGES_FILE)

manualctrl.update()

if is_volume_mounted(INTERNAL_VOL):
    create_tmadmin_share()
    start_tmnas_service()
